Package: nicheR
Type: Package
Title: Ellipsoid-Based Virtual Niches and Visualization
Version: 0.1.0
Authors@R: c(
    person(
      "Mariana", "Castaneda-Guzman",
      email = "marianacg@vt.edu", 
      role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0001-6106-4284")
    ),
    person(
      "Connor", "Hughes",
      email = "connorjh@vt.edu", 
      role = "aut",
      comment = c(ORCID = "0000-0002-3720-0837")
    ),
    person(
      "Paanwaris", "Paansri",
      email = "paanwaris@vt.edu", 
      role = "aut",
      comment = c(ORCID = "0000-0001-9992-098X")
    ),
    person(
      "Marlon E.", "Cobos",
      email = "manubio13@gmail.com", 
      role = "aut",
      comment = c(ORCID = "0000-0002-2611-1767")
    )
  )
Description: Provides a robust set of tools for researchers and modelers to
    construct and define virtual ecological niches using ellipsoid geometries.
    It enables the identification and extraction of suitable environmental
    areas, simulation of species occurrence points with various sampling
    strategies, and visualization of niche boundaries and simulated occurrences
    in both environmental and geographic space. Inspired by methodologies in
    'NicheA' and the 'virtualspecies' R package, 'nicheR' aims to streamline
    the process of niche conceptualization and data generation for ecological
    studies. Methodological and theoretical foundations are described in
    Peterson et al. (2011, ISBN:9780691136882),
    Etherington et al. (2009) <doi:10.1111/j.1365-2699.2008.02041.x>,
    Qiao et al. (2015) <doi:10.1111/ecog.01961>,
    Nunez-Penichet et al. (2021) <doi:10.21425/F5FBG52142>,
    Cobos and Peterson (2022) <doi:10.17161/bi.v17i.15985>,
    Alkishe et al. (2022) <doi:10.5194/we-22-33-2022>,
    and Leroy et al. (2015) <doi:10.1111/ecog.01388>.
URL: https://github.com/castanedaM/nicheR,
        https://castanedam.github.io/nicheR/
BugReports: https://github.com/castanedaM/nicheR/issues
Depends: R (>= 3.5)
Imports: terra
Suggests: rgl (>= 1.3), knitr, rmarkdown
VignetteBuilder: knitr
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-08 16:02:15 UTC; mcguz
Author: Mariana Castaneda-Guzman [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6106-4284>),
  Connor Hughes [aut] (ORCID: <https://orcid.org/0000-0002-3720-0837>),
  Paanwaris Paansri [aut] (ORCID:
    <https://orcid.org/0000-0001-9992-098X>),
  Marlon E. Cobos [aut] (ORCID: <https://orcid.org/0000-0002-2611-1767>)
Maintainer: Mariana Castaneda-Guzman <marianacg@vt.edu>
Repository: CRAN
Date/Publication: 2026-06-16 19:30:08 UTC
Built: R 4.5.2; ; 2026-06-17 00:20:07 UTC; unix
