# Generated by roxygen2: do not edit by hand

S3method(predict,nicheR_community)
S3method(predict,nicheR_ellipsoid)
S3method(print,nicheR_community)
S3method(print,nicheR_ellipsoid)
export(add_data)
export(add_data_3d)
export(add_ellipsoid)
export(add_ellipsoid_3d)
export(apply_bias)
export(build_ellipsoid)
export(conserved_ellipses)
export(covariance_limits)
export(ellipsoid_calculator)
export(ellipsoid_volume)
export(nested_ellipses)
export(new_nicheR_community)
export(plot_community)
export(plot_ellipsoid)
export(plot_ellipsoid_3d)
export(plot_ellipsoid_pairs)
export(prepare_bias)
export(random_ellipses)
export(ranges_from_data)
export(ranges_from_stats)
export(read_nicheR)
export(sample_biased_data)
export(sample_data)
export(save_nicheR)
export(update_covariance)
export(update_ellipsoid_covariance)
export(virtual_data)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rainbow)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(stats,complete.cases)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(terra,app)
importFrom(terra,compareGeom)
importFrom(terra,crop)
importFrom(terra,global)
importFrom(terra,mask)
importFrom(terra,nlyr)
importFrom(terra,rast)
importFrom(terra,resample)
importFrom(utils,combn)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
